@@ -193,7 +193,9 @@ def loop (timeout = 30.0):
 
 	_polling = 0
 
-poll_fun = asyncore.poll if '__pypy__' in sys.builtin_module_names else asyncore.epoll
+poll_fun = asyncore.poll
+if os.name == 'posix' and '__pypy__' not in sys.builtin_module_names:
+	poll_fun = asyncore.epoll
 
 def remove_notsocks (map):
 	global _select_errors, _logger
